function estLabels = nsClassify(Xtest,Ufull)
% Nearest-subspace classifier
% 
% Syntax estLabels = nsClassify(Xtest,Ufull)
%
% Inputs:       Xtest is a D x N matrix of test data
%               Ufull is a D x r x K matrix where Ufull(:,:,k) is a basis for
%               the subspace that best represents class k
%
% Outputs:      estLabels is an N x 1 vector of estimated class labels

[D,N] = size(Xtest);
K = size(Ufull,3);

fullResids = zeros(N,K);
for kk = 1:K
    Uk = squeeze(Ufull(:,:,kk));
    resids = Xtest - Uk*(Uk'*Xtest);
    fullResids(:,kk) = sqrt(sum(resids.^2)); 
end
[~,estLabels] = min(fullResids,[],2);