"""
Nearest Subspace Classifier demo - classifyDigits

ECE 510

python version: 3.7.2

Spring 2019
"""

import numpy as np
import matplotlib.pyplot as plt
from trainUoS import trainUoS
from nsClassify import nsClassify
from scipy.io import loadmat

## Load data: 
data = loadmat("mnistSubset.mat")

## Task 1 - Raise your hand when finished
X = data['X']
trueLabels = data['trueLabels'][:,0]

# Plot the digits
classes = [0, 4, 9]
figs, axes = plt.subplots(3, 3, figsize=(12, 8))
plt.set_cmap('gray')
axes = axes.ravel()
figInd = 0
for ii in classes:
    Xtemp = X[:, ii + ii * 200].reshape(28,28)
    axes[figInd].imshow(Xtemp.T)
    figInd+=1
    Xtemp = X[:, ii + 50 + ii * 200].reshape(28,28)
    axes[figInd].imshow(Xtemp.T)
    figInd+=1
    Xtemp = X[:, ii + 100 + ii * 200].reshape(28,28)
    axes[figInd].imshow(Xtemp.T)
    figInd+=1

plt.suptitle('Example Images')
plt.show()

# Look at some singular values
plt.figure()
for ii in classes:
    # Xi = X[:, np.argwhere(trueLabels==ii+1)][:,:,0]
    Xi = X[:, trueLabels==ii+1]
    _, s, _ = np.linalg.svd(Xi, full_matrices=False)
    plt.plot(s, '-x', label=ii)

plt.title('Singular Values')
plt.legend()
plt.show()

# Plot requested ratio
plt.figure()
for ii in classes:
    Xi = X[:, trueLabels==ii+1]
    _, s, _ = np.linalg.svd(Xi, full_matrices=False)
    plt.plot(np.cumsum(s) / np.sum(s), '-x', label=ii)

plt.title('Relative energy in singular values 1:K')
plt.legend()
plt.show()

## Task 2 - Raise your hand when finished
# Number of classes
K = 10 

Xtrain = np.array([]).reshape(784, 0)
ytrain = np.array([]).reshape(0, 1)
Xtest = np.array([]).reshape(784, 0)
ytest = np.array([]).reshape(0, 1)
for kk in range(1,K+1):
    Xk = X[:, np.argwhere(trueLabels==kk)[:,0]]
    Xtrain = np.append(Xtrain, Xk[:, :100], axis=1)
    Xtest = np.append(Xtest, Xk[:, 100:], axis=1)
    ytrain = np.append(ytrain, kk*np.ones([100, 1]), axis=0)
    ytest = np.append(ytest, kk*np.ones([100, 1]), axis=0)

# Tunning parameter
r = 4
Ufull = trainUoS(Xtrain, ytrain, r)

# Plot a few principal components
Uk = Ufull[:,:,5]

figs, axes = plt.subplots(1, 4, figsize=(12,3))
plt.set_cmap('gray')
axes[0].imshow(Uk[:,0].reshape(28,28).T)
axes[1].imshow(Uk[:,1].reshape(28,28).T)
axes[2].imshow(Uk[:,2].reshape(28,28).T)
axes[3].imshow(Uk[:,3].reshape(28,28).T)
plt.suptitle('Principal Components')
plt.show()

## Task 3 - Raise your hand when you finished

# Error for a variety of r values
Nr = 20
errs = np.zeros([Nr, 1])
for rr in range(Nr):
    Ufull = trainUoS(Xtrain, ytrain, rr+1)
    estLabels = nsClassify(Xtest,Ufull)
    errs[rr] = np.sum(estLabels != ytest)/len(ytest)
    print(errs[rr])

# Show some mislabeled examples
errPoints = np.argwhere(estLabels != ytest)[:,0]
Np = 64
inds = errPoints[np.round(np.linspace(0,len(errPoints)-1,Np)).astype(int)]

figs, axes = plt.subplots(8, 8, figsize=(12,12))
axes = axes.ravel()
plt.set_cmap('gray')
for ii in np.arange(Np):
    Xi = Xtest[:,inds[ii]].reshape(28,28)
    axes[ii].imshow(Xi.T)
    axes[ii].axis('off')

plt.suptitle('Mislabeled Examples')
plt.show()
