"""
Nearest Subspace Classifier demo - nsClassify

ECE 510

python version: 3.7.2

Spring 2019
"""

import numpy as np

def nsClassify(Xtest, Ufull):
    r"""
    Nearest-subspace classifier

    Syntax: estLabels = nsClassify(Xtest,Ufull)

    Inputs:
    :param Xtest: A D x N matrix of test data
    :param Ufull: Ufull is a D x r x K matrix where Ufull(:,:,k) is a basis for
    the subspace that best represents class k

    Output:
    estLabels is an N x 1 vector of estimated class labels
    """
    D, N = np.shape(Xtest)
    _, r, _ = np.shape(Ufull)
    K = np.shape(Ufull)[2]

    fullResids = np.zeros([N, K])
    
    for kk in range(K):
        Uk = np.squeeze(Ufull[:, :, kk]).reshape(D, r)
        resids = Xtest - Uk @ (Uk.T @ Xtest)
        fullResids[:, kk] = np.sqrt(np.sum(resids**2, axis=0))
    
    estLabels = np.argmin(fullResids, 1).reshape(N, 1)+ np.ones((N, 1))

    return estLabels    