"""
Nearest Subspace Classifier demo - trainUoS

ECE 510

python version: 3.7.2

Spring 2019
"""

import numpy as np
from scipy.sparse.linalg import svds

def trainUoS(X, y, r):
    r"""
    Learn union of r - dimensional subspaces that best fit a dataset from K 
    classes. 
    
    Syntax: Ufull = trainUoS(X,y,r)

    Inputs:
    :param X: A D x N matrix of training data
    :param y: y is an N x 1 vectors of labels
    :param r: r is the rank/dimension of the subspaces

    Output:  
    Ufull is a D x r x K matrix where Ufull(:,:,k) is a basis for
    the subspace that best represents class k
    """

    D, N = X.shape
    K = int(np.max(y))

    Ufull = np.zeros([D, r, K])

    for kk in range(K):
        Xk = X[:, np.argwhere(y==kk+1)[:,0]]
        Ufull[:, :, kk], _, _ = svds(Xk, k=r)
    return Ufull
